@@ -5,7 +5,8 @@ divide_matrices <- function(data, fixed, random = NULL, analysis_type,
                             models = NULL,  timevar = NULL, no_model = NULL,
                             nonprop = NULL, rev = NULL,
                             ppc = TRUE, shrinkage = FALSE,
-                            warn = TRUE, mess = TRUE, df_basehaz = 6, ...) {
+                            warn = TRUE, mess = TRUE, df_basehaz = 6,
+                            rd_vcov = rd_vcov, ...) {
 
   # id's and groups ------------------------------------------------------------
   # extract the id variable from the random effects formula and get groups
@@ -88,7 +89,7 @@ divide_matrices <- function(data, fixed, random = NULL, analysis_type,
     if (length(add_to_aux) > 0)
       as.formula(paste("~", paste0(add_to_aux, collapse = " + ")))
   } else {
-    as.formula(paste0(c(deparse(auxvars, width.cutoff = 500), add_to_aux),
+    as.formula(paste0(c(deparse(auxvars, width.cutoff = 500L), add_to_aux),
                       collapse = " + "))
   }
 
@@ -239,6 +240,10 @@ divide_matrices <- function(data, fixed, random = NULL, analysis_type,
                            groups[[gsub("M_", "", k)]]), , drop = FALSE]
   }
 
+
+  nranef <- get_nranef(idvar = idvar, random = random, data = data)
+  rd_vcov <- check_rd_vcov(rd_vcov = rd_vcov, nranef = nranef)
+
   list(data = data,
        fixed = fixed, random = random, models = models,
 
@@ -259,6 +264,7 @@ divide_matrices <- function(data, fixed, random = NULL, analysis_type,
        ppc = ppc,
        shrinkage = shrinkage,
        df_basehaz = df_basehaz,
+       rd_vcov = rd_vcov,
 
        scale_pars = scale_pars,
        outcomes = outcomes,
