@@ -1,6 +1,4 @@
-context("GLMM")
 library("JointAI")
-library("splines")
 
 Sys.setenv(IS_CHECK = "true")
 if (identical(Sys.getenv("NOT_CRAN"), "true")) {
@@ -19,7 +17,6 @@ if (identical(Sys.getenv("NOT_CRAN"), "true")) {
 
 
   run_glmm_models <- function() {
-    cat("\nRunning glmm models...\n")
     sink(tempfile())
     on.exit(sink())
     invisible(force(suppressWarnings({
@@ -45,10 +42,10 @@ if (identical(Sys.getenv("NOT_CRAN"), "true")) {
                         data = longDF, n.adapt = 5, n.iter = 10, seed = 2020,
                         warn = FALSE, mess = FALSE),
         m0b3 = glme_imp(b1 ~ 1 + (1 | id), family = binomial(link = "log"),
-                        data = longDF, n.adapt = 5, n.iter = 10, seed = 2020,
+                        data = longDF, n.adapt = 50, n.iter = 10, seed = 2020,
                         warn = FALSE, mess = FALSE),
         m0b4 = glme_imp(b1 ~ 1 + (1 | id), family = binomial(link = "cloglog"),
-                        data = longDF, n.adapt = 5, n.iter = 10, seed = 2020,
+                        data = longDF, n.adapt = 50, n.iter = 10, seed = 2020,
                         warn = FALSE, mess = FALSE),
 
         m0c1 = glme_imp(L1 ~ 1 + (1 | id), family = Gamma(link = "inverse"),
@@ -341,34 +338,37 @@ if (identical(Sys.getenv("NOT_CRAN"), "true")) {
 
   test_that("data_list remains the same", {
     skip_on_cran()
-    print_output(lapply(models, "[[", "data_list"), type = "value")
+    print_output(lapply(models, "[[", "data_list"), type = "value",
+                 context = "glmm")
   })
 
   test_that("jagsmodel remains the same", {
     skip_on_cran()
-    print_output(lapply(models, "[[", "jagsmodel"))
+    print_output(lapply(models, "[[", "jagsmodel"), context = "glmm")
   })
 
   test_that("GRcrit and MCerror give same result", {
     skip_on_cran()
-    print_output(lapply(models0, GR_crit, multivariate = FALSE))
-    print_output(lapply(models0, MC_error))
+    print_output(lapply(models0, GR_crit, multivariate = FALSE), context = "glmm")
+    print_output(lapply(models0, MC_error), context = "glmm")
   })
 
 
   test_that("summary output remained the same", {
     skip_on_cran()
-    print_output(lapply(models0, print))
-    print_output(lapply(models0, coef))
-    print_output(lapply(models0, confint))
-    print_output(lapply(models0, summary, missinfo = TRUE))
-    print_output(lapply(models0, function(x) coef(summary(x))))
+    print_output(lapply(models0, print), context = "glmm")
+    print_output(lapply(models0, coef), context = "glmm")
+    print_output(lapply(models0, confint), context = "glmm")
+    print_output(lapply(models0, summary, missinfo = TRUE), context = "glmm")
+    print_output(lapply(models0, function(x) coef(summary(x))),
+                 context = "glmm")
   })
 
 
 
   test_that("prediction works", {
     for (k in seq_along(models)) {
+
       expect_s3_class(predict(models[[k]], type = "link", warn = FALSE)$fitted,
                       "data.frame")
       expect_s3_class(predict(models[[k]], type = "response",
@@ -389,21 +389,21 @@ if (identical(Sys.getenv("NOT_CRAN"), "true")) {
 
 
   test_that("residuals", {
-    for (k in seq_along(models)) {
-      expect_is(residuals(models[[k]], type = "response", warn = FALSE),
-                "numeric")
+    for (k in seq_along(models)[8]) {
+      expect_type(residuals(models[[k]], type = "response", warn = FALSE),
+                  "double")
 
       if (models[[k]]$analysis_type %in% c("beta", "glmm_beta")) {
         expect_error(residuals(models[[k]], type = "working", warn = FALSE))
         expect_error(residuals(models[[k]], type = "pearson", warn = FALSE))
       } else {
-        expect_is(residuals(models[[k]], type = "working", warn = FALSE),
-                  "numeric")
-        expect_is(residuals(models[[k]], type = "pearson", warn = FALSE),
-                  "numeric")
+        expect_type(residuals(models[[k]], type = "working", warn = FALSE),
+                    "double")
+        expect_type(residuals(models[[k]], type = "pearson", warn = FALSE),
+                    "double")
       }
     }
-    expect_is(residuals(models$m5a, warn = FALSE), "numeric")
+    expect_type(residuals(models$m5a, warn = FALSE), "double")
   })
 
 
